package authdiscord

import (
	"fmt"

	configserver "example.com/config/server"

	"github.com/ravener/discord-oauth2"
	"golang.org/x/oauth2"
)

type DiscordUser struct {
	Id                     string  `json:"id"`
	Username               string  `json:"username"`
	Avatar                 string  `json:"avatar"`
	Discriminator          string  `json:"discriminator"`
	Public_Flags           float64 `json:"public_flags"`
	Flags                  float64 `json:"flags"`
	Banner                 string  `json:"banner"`
	Accent_Color           float64 `json:"accent_color"`
	Global_Name            string  `json:"global_name"`
	Avatar_Decoration_Data struct {
		Asset      string `json:"asset"`
		Sku_Id     string `json:"sku_id"`
		Expires_At string `json:"expired_at"`
	} `json:"avatar_decoration_data"`
	Collectibles  string  `json:"collectibles"`
	Banner_Color  string  `json:"banner_color"`
	Clan          string  `json:"clan"`
	Primary_Guild string  `json:"primary_guild"`
	Mfa_Enabled   bool    `json:"mfa_enabled"`
	Locale        string  `json:"locale"`
	Premium_Type  float64 `json:"premium_type"`
}

func CreateDiscordOAuthConfig(config configserver.AppConfig) *oauth2.Config {
	protocolString := ""
	if config.API.Https {
		protocolString = "https://"
		config.API.Port = ""
	} else {
		protocolString = "http://"
		config.API.Port = fmt.Sprintf(":%s", config.API.Port)
	}
	redirectUrlString := fmt.Sprintf("%s%s%s/auth/callback", protocolString, config.API.Domain, config.API.Port)
	oauthconfig := &oauth2.Config{
		ClientID:     config.OAuth.ClientID,
		ClientSecret: config.OAuth.ClientSecret,
		RedirectURL:  redirectUrlString,
		Scopes:       []string{discord.ScopeIdentify},
		Endpoint:     discord.Endpoint,
	}
	return oauthconfig
}
