package databasemodels

import "gorm.io/gorm"

type User struct {
	gorm.Model
	Id               string
	DisplayName      string
	Username         string
	Avatar           string
	AvatarDecoration string
	LoginToken       string
	LoggedIn         bool
}

type Person struct {
	gorm.Model
	Name   string
	Groups []Group // Unique
}

type Group struct {
	gorm.Model
	Name string `json:"name"`
}

type Character struct {
	gorm.Model
	Name         string
	Owners       []Person // Unique
	Roles        []Role   // Unique
	FunctionSets []FunctionSet
	Inventory    []InventorySlot
}

type Role struct {
	gorm.Model
	Name       string
	Tiers      []Tier
	Visibility []Group // Unique
}

type Tier struct {
	gorm.Model
	FunctionSets []FunctionSet
}

type FunctionSet struct {
	gorm.Model
	Functions []Function
}

type Function struct {
	gorm.Model
	Name         string
	Tags         FunctionTag
	Requirements []Function
}

type FunctionTag struct {
	gorm.Model
	Name string
}

type InventorySlot struct {
	gorm.Model
	Item     Item
	Quantity int64 // Positive
}

type Item struct {
	gorm.Model
	Name                string
	Functions           []Function
	FlavorText          string
	RulesDescription    string
	PhysrepRequirements string
	Tags                []ItemTag // Unique
	Customizations      []Customization
	Visibility          []Group // Unique
}

type ItemTag struct {
	gorm.Model
	Naem string
}

type Customization struct {
	gorm.Model
	Name                string
	Functions           []Function
	FlavorText          string
	RulesDescription    string
	PhysrepRequirements string
	Tags                []ItemTag // Unique
	Visibility          []Group   // Unique
}

type Schematic struct {
	gorm.Model
	Material     []InventorySlot
	Tools        []InventorySlot
	Requirements []Function
	TimeUnits    int64 // Positive
	Result       InventorySlot
	Visibility   []Group // Unique
}
